#include <fcntl.h>
#include <inttypes.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/mman.h>
#include <unistd.h>


int main(int argc, char const* argv[])
{
    int gpiofd = open("/dev/gpiomem", O_RDWR);
    if (gpiofd < 0) {
        perror("Cannot open /dev/gpiomem");
        exit(EXIT_FAILURE);
    }

    void* gpiomap = mmap(NULL, 4096, PROT_READ | PROT_WRITE, MAP_SHARED, gpiofd, 0);
    if (gpiomap == MAP_FAILED) {
        perror("Cannot mmap gpio opened from /dev/mem");
        exit(EXIT_FAILURE);
    }

    /* GPIO 13 is controlled by function select register 1, at offset 0x4. */
    uint32_t* const fselreg1 = (uint32_t*) (gpiomap + 0x04);
    /* Bits 9-11 in function select register 1 determine the function of pin
     * 13. The bits 001 mean output pin.*/
    *fselreg1 &= ~(0x7 << 9);
    *fselreg1 |= (1 << 9);

    /* Setting a bit in output clear register sets the pin low. Setting a bit
     * in the output set register sets the pin high. */
    uint32_t* const outputset0 = (uint32_t*) (gpiomap + 0x1c);
    uint32_t* const outputclr0 = (uint32_t*) (gpiomap + 0x28);
    *outputset0 |= (1 << 13);
    usleep(200000);
    *outputclr0 |= (1 << 13);
}
