const uint8_t bits = 8;
const uint32_t speed = 100000;

/** Open the first spidev device, i.e., /dev/spidev0.0 and return its file
 * descriptor. After open() the device is initalized to 8-bit words and 100 kHz
 * speed. */
static int open_spi() {
    int ret;
    int spifd = open("/dev/spidev0.0", O_RDWR);
    if (spifd < 0)
        pabort("Cannot open /dev/spidev0.0");

    ret = ioctl(spifd, SPI_IOC_WR_BITS_PER_WORD, &bits);
    if (ret < 0)
        pabort("Cannot set SPI word size");
    ret = ioctl(spifd, SPI_IOC_WR_MAX_SPEED_HZ, &speed);
    if (ret < 0)
        pabort("Cannot set SPI max speed");

    return spifd;
}

/* Transceive a byte over SPI device at file descriptor spifd. That is,
 * transfer len bytes of the array tx and receive as many bytes and store them
 * in rx. See open_spi() to open an spidev device. */
static int send_spi_message(int spifd, uint8_t* tx, uint8_t* rx, size_t len) {
    struct spi_ioc_transfer tr = {
        .tx_buf = (unsigned long) tx,
        .rx_buf = (unsigned long) rx,
        .len = len,
        .delay_usecs = 0,
        .speed_hz = speed,
        .bits_per_word = bits,
    };

    return ioctl(spifd, SPI_IOC_MESSAGE(1), &tr);
}
